% Creating an output structure similar to the input
OutputPort1 = InputPort1;
OutputPort2 = InputPort1;

% Switch 2 (Open Flow Table)
OFT2_SwitchPort = ' ';
OFT2_VLANID = ' ';
OFT2_PCP = ' ';
OFT2_MACSA = ' ';
OFT2_MACDA = 'BBBBBBBBBBBB';
OFT2_EthType = ' ';
OFT2_IPSource = ' ';
OFT2_IPDest = ' ';  
OFT2_TOS = ' ';
OFT2_Protocol = ' ';
OFT2_L4SP = ' ';  
OFT2_L4DP = ' ';

% Read in frame header data
for m = 1 : (22*8)
    BinSeqInput(m) = InputPort1.Sequence(m);

end

% Convert to string (no spaces)
BinStrInput = num2str(BinSeqInput);
BinStrInput = BinStrInput(~isspace(BinStrInput));

% Convert from binary to hexadecimal
i = length(BinStrInput);
n = ceil(i/4); 

for g = n : -1 : 1
    if i > 4
        HexStrInput(g) = b2h(BinStrInput(i-3 : i));
        i = i - 4;
    else
        HexStrInput(g) = b2h(BinStrInput(1 : i));
    end
end

% Allocate to Eth frame fields
Pre_S2 = HexStrInput(1:14);
SFD_S2 = HexStrInput(15:16);
DA_S2 = HexStrInput(17:28);
SA_S2 = HexStrInput(29:40);
EType_S2 = HexStrInput(41:44);

% Determine length of client payload (octets)
PayloadOctetLength = (length(InputPort1.Sequence)-22*8)/8;

% Actions (Instruction(s) from OF Controller)
% Import data from Flow Controller message
fileID = fopen('OF2.txt');
C2 = textscan(fileID,'%s %s');
fclose(fileID);

% Actions (Instruction(s) from OF Controller)
% Read in data 
DestMAC2 = C2{2}{1};
Inst2 = C2{2}{2};
% Forward incoming packet to output port (P5) if there is a match with flow
% table
if strcmp (DA_S2,DestMAC2)
     Action = Inst2;
else
     Action = 'None';
end

if strcmp(Action, Inst2)
    % Allocate binary sequence to output port (Frame data)
    for m = 1 : (22*8)
        OutputPort1.Sequence(m) = InputPort1.Sequence(m);
    end
    
    for x = (22*8 + 1): length(InputPort1.Sequence)
        OutputPort1.Sequence(x) = 0;
    end
       
    % Forward binary sequence (client payload) to Host PC B
    for m = 1 : (22*8)
        OutputPort2.Sequence(m) = 0;
    end
    
    for x = (22*8 + 1): length(InputPort1.Sequence)
        OutputPort2.Sequence(x) = InputPort1.Sequence(x);
    end
else
    % Allocate binary sequence to output ports (empty)
    for m = 1 : length(InputPort1.Sequence)
        OutputPort1.Sequence(m) = 0;
        OutputPort2.Sequence(m) = 0;
    end
    PayloadOctetLength = 0;
end

% Print results to text file
fid=fopen('C:\Users\mabouseif\Desktop\Mohamed Abouseif\OptiSys_issues\Rakesh_IndianMatlab\Ethernet Switch and OF 27 May 16\Ethernet Switch and OF 27 May 16\ResultsEthFrame.txt','w');
fprintf(fid,'Pre: %s \r\n', Pre_S2);
fprintf(fid,'SFD: %s \r\n', SFD_S2);
fprintf(fid,'Dest MAC: %s \r\n', DA_S2);
fprintf(fid,'Src MAC: %s \r\n', SA_S2);
fprintf(fid,'Eth Type: %s \r\n', EType_S2);
fprintf(fid,'Client payload size (octets): %s', num2str(PayloadOctetLength));
fclose(fid);
